# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import transaction
from django.http import HttpResponse
from django.shortcuts import redirect, render
from django_logit import logit
from django_we.models import (ComponentAuthTokenRefreshLogInfo, ComponentTokenRefreshLogInfo,
                              ComponentVerifyTicketLogInfo, TicketRefreshLogInfo, TokenRefreshLogInfo)
from furl import furl
from json_response import auto_response
from pywe_component_authorizer_token import authorizer_access_token, initial_authorizer_access_token
from pywe_component_ticket import set_component_verify_ticket
from pywe_decrypt import msg
from pywe_jssdk import jsapi_signature_params
from pywe_oauth import get_access_info, get_oauth_code_url, get_oauth_redirect_url, get_userinfo
from pywe_qrcode import qrcode_create
from pywe_sign import check_callback_signature
from pywe_storage import RedisStorage
from pywe_token import access_token, refresh_access_token
from pywe_xml import xml_to_dict


JSAPI = settings.WECHAT.get(getattr(settings, 'DJANGO_WE_OAUTH_CFG') if hasattr(settings, 'DJANGO_WE_OAUTH_CFG') else 'JSAPI', {})


def final_cfg(request, state=None):
    CFG = JSAPI
    if hasattr(settings, 'DJANGO_WE_CFG_FUNC') and hasattr(settings.DJANGO_WE_CFG_FUNC, '__call__'):
        CFG = settings.DJANGO_WE_CFG_FUNC(request, state) or JSAPI
    return CFG


def quote_state(request, state=None):
    if hasattr(settings, 'DJANGO_WE_QUOTE_OR_NOT') and not getattr(settings, 'DJANGO_WE_QUOTE_OR_NOT'):
        return state
    if hasattr(settings, 'DJANGO_WE_QUOTE_STATE_FUNC') and hasattr(settings.DJANGO_WE_QUOTE_STATE_FUNC, '__call__'):
        state = settings.DJANGO_WE_QUOTE_STATE_FUNC(request, state)
    return state


def unquote_state(request, state=None):
    if hasattr(settings, 'DJANGO_WE_QUOTE_OR_NOT') and not getattr(settings, 'DJANGO_WE_QUOTE_OR_NOT'):
        return state
    if hasattr(settings, 'DJANGO_WE_UNQUOTE_STATE_FUNC') and hasattr(settings.DJANGO_WE_UNQUOTE_STATE_FUNC, '__call__'):
        state = settings.DJANGO_WE_UNQUOTE_STATE_FUNC(request, state)
    if not state and hasattr(settings, 'WECHAT_DEFAULT_REDIRECT_URI'):
        state = settings.WECHAT_DEFAULT_REDIRECT_URI
    return state


def final_oauth_uri(request, state=None):
    oauth_uri = hasattr(settings, 'WECHAT_OAUTH2_REDIRECT_URI') and settings.WECHAT_OAUTH2_REDIRECT_URI or ''
    if hasattr(settings, 'DJANGO_WE_OAUTH2_REDIRECT_URI_FUNC') and hasattr(settings.DJANGO_WE_OAUTH2_REDIRECT_URI_FUNC, '__call__'):
        oauth_uri = settings.DJANGO_WE_OAUTH2_REDIRECT_URI_FUNC(request, state)
    return oauth_uri


def final_direct_userinfo_redirect_uri(request):
    redirect_uri = hasattr(settings, 'WECHAT_DIRECT_USERINFO_REDIRECT_URI') and settings.WECHAT_DIRECT_USERINFO_REDIRECT_URI or ''
    if hasattr(settings, 'DJANGO_WE_DIRECT_USERINFO_REDIRECT_URI_FUNC') and hasattr(settings.DJANGO_WE_DIRECT_USERINFO_REDIRECT_URI_FUNC, '__call__'):
        redirect_uri = settings.DJANGO_WE_DIRECT_USERINFO_REDIRECT_URI_FUNC(request)
    return redirect_uri


def final_direct_base_redirect_uri(request):
    redirect_uri = hasattr(settings, 'WECHAT_DIRECT_BASE_REDIRECT_URI') and settings.WECHAT_DIRECT_BASE_REDIRECT_URI or ''
    if hasattr(settings, 'DJANGO_WE_DIRECT_BASE_REDIRECT_URI_FUNC') and hasattr(settings.DJANGO_WE_DIRECT_BASE_REDIRECT_URI_FUNC, '__call__'):
        redirect_uri = settings.DJANGO_WE_DIRECT_BASE_REDIRECT_URI_FUNC(request)
    return redirect_uri


def final_userinfo_redirect_uri(request):
    redirect_uri = hasattr(settings, 'WECHAT_USERINFO_REDIRECT_URI') and settings.WECHAT_USERINFO_REDIRECT_URI or ''
    if hasattr(settings, 'DJANGO_WE_USERINFO_REDIRECT_URI_FUNC') and hasattr(settings.DJANGO_WE_USERINFO_REDIRECT_URI_FUNC, '__call__'):
        redirect_uri = settings.DJANGO_WE_USERINFO_REDIRECT_URI_FUNC(request)
    return redirect_uri


def final_base_redirect_uri(request):
    redirect_uri = hasattr(settings, 'WECHAT_BASE_REDIRECT_URI') and settings.WECHAT_BASE_REDIRECT_URI or ''
    if hasattr(settings, 'DJANGO_WE_BASE_REDIRECT_URI_FUNC') and hasattr(settings.DJANGO_WE_BASE_REDIRECT_URI_FUNC, '__call__'):
        redirect_uri = settings.DJANGO_WE_BASE_REDIRECT_URI_FUNC(request)
    return redirect_uri


def redis_storage(request):
    r = None

    if hasattr(settings, 'WECHAT_REDIS_OBJ'):
        r = settings.WECHAT_REDIS_OBJ

    if hasattr(settings, 'DJANGO_WE_REDIS_OBJ_FUNC') and hasattr(settings.DJANGO_WE_REDIS_OBJ_FUNC, '__call__'):
        r = settings.DJANGO_WE_REDIS_OBJ_FUNC(request) or r

    return r and RedisStorage(r)


# Token/Ticket Store into Database after Fetched/Refreshed
def token_fetched_func(appid, secret, access_info):
    if not hasattr(settings, 'DJANGO_WE_TOKEN_STORE_OR_NOT') or getattr(settings, 'DJANGO_WE_TOKEN_STORE_OR_NOT'):
        TokenRefreshLogInfo.objects.create(appid=appid, secret=secret, access_info=access_info)


def ticket_fetched_func(appid, secret, ticket_type, ticket_info):
    if not hasattr(settings, 'DJANGO_WE_TICKET_STORE_OR_NOT') or getattr(settings, 'DJANGO_WE_TICKET_STORE_OR_NOT'):
        TicketRefreshLogInfo.objects.create(appid=appid, secret=secret, ticket_type=ticket_type, ticket_info=ticket_info)


def component_token_fetched_func(appid, secret, access_info):
    if not hasattr(settings, 'DJANGO_WE_COMPONENT_TOKEN_STORE_OR_NOT') or getattr(settings, 'DJANGO_WE_COMPONENT_TOKEN_STORE_OR_NOT'):
        ComponentTokenRefreshLogInfo.objects.create(component_appid=appid, component_secret=secret, component_access_info=access_info)


def component_auth_token_fetched_func(component_appid, component_secret, authorizer_appid, component_authorizer_access_info):
    if not hasattr(settings, 'DJANGO_WE_COMPONENT_AUTH_TOKEN_STORE_OR_NOT') or getattr(settings, 'DJANGO_WE_COMPONENT_AUTH_TOKEN_STORE_OR_NOT'):
        ComponentAuthTokenRefreshLogInfo.objects.create(
            component_appid=component_appid,
            component_secret=component_secret,
            authorizer_appid=authorizer_appid,
            component_authorizer_access_info=component_authorizer_access_info
        )


def component_verify_ticket_push_func(component_appid, component_secret, component_verify_ticket):
    if not hasattr(settings, 'DJANGO_WE_COMPONENT_VERIFY_TICKET_STORE_OR_NOT') or getattr(settings, 'DJANGO_WE_COMPONENT_VERIFY_TICKET_STORE_OR_NOT'):
        ComponentVerifyTicketLogInfo.objects.create(
            component_appid=component_appid,
            component_secret=component_secret,
            component_verify_ticket=component_verify_ticket
        )


def we_oauth2(request):
    scope = request.GET.get('scope', '') or request.GET.get('s', 'snsapi_userinfo')
    redirect_url = request.GET.get('redirect_url', '') or request.GET.get('r', '/')
    default_url = request.GET.get('default_url', '') or request.GET.get('d', '')
    direct_redirect = bool(request.GET.get('direct_redirect', '') or request.GET.get('dr', ''))

    if not (redirect_url or default_url):
        return render(request, 'django_we/errmsg.html', {'title': 'Error', 'errmsg': 'Redirect or Default URL Should Exists'})

    if request.wechat:
        CFG = final_cfg(request, redirect_url)
        if direct_redirect:
            redirect_uri = final_direct_userinfo_redirect_uri(request) if scope == 'snsapi_userinfo' else final_direct_base_redirect_uri(request)
        else:
            redirect_uri = final_userinfo_redirect_uri(request) if scope == 'snsapi_userinfo' else final_base_redirect_uri(request)
        return redirect(get_oauth_code_url(CFG['appID'], redirect_uri, scope, quote_state(request, redirect_url)))

    return redirect(default_url or redirect_url)


@transaction.atomic
def base_redirect(request):
    code = request.GET.get('code', '')
    state = request.GET.get('state', '')

    final_state = unquote_state(request, state)

    CFG = final_cfg(request, final_state)

    access_info = get_access_info(CFG['appID'], CFG['appsecret'], code)
    if 'errcode' in access_info:
        return redirect(get_oauth_redirect_url(final_oauth_uri(request, final_state), 'snsapi_base', final_state))

    query_params = {}

    # Toset Cookie When ``DJANGO_WE_BASE_REDIRECT_SET_COOKIE = True``
    if hasattr(settings, 'DJANGO_WE_BASE_REDIRECT_SET_COOKIE') and getattr(settings, 'DJANGO_WE_BASE_REDIRECT_SET_COOKIE'):
        if hasattr(settings, 'DJANGO_WE_BASE_COOKIE_FUNC') and hasattr(settings.DJANGO_WE_BASE_COOKIE_FUNC, '__call__'):
            query_params, cookie_key, cookie_value = settings.DJANGO_WE_BASE_COOKIE_FUNC(code, final_state, access_info)

            response = redirect(furl(final_state).remove(access_info.keys()).add(access_info).remove(query_params.keys()).add(query_params).url)

            if hasattr(settings, 'DJANGO_WE_BASE_SET_COOKIE_FUNC') and hasattr(settings.DJANGO_WE_BASE_SET_COOKIE_FUNC, '__call__'):
                return settings.DJANGO_WE_BASE_SET_COOKIE_FUNC(code, final_state, access_info, query_params, cookie_key, cookie_value)

            max_age = hasattr(settings, 'DJANGO_WE_COOKIE_MAX_AGE') and getattr(settings, 'DJANGO_WE_COOKIE_MAX_AGE') or 30 * 24 * 60 * 60  # 30d
            cookie_salt = hasattr(settings, 'DJANGO_WE_COOKIE_SALT') and getattr(settings, 'DJANGO_WE_COOKIE_SALT') or 'djwe'  # Salt for ``set_signed_cookie``

            response.set_signed_cookie(cookie_key, cookie_value, salt=cookie_salt, **{
                'max_age': max_age,
                'httponly': True,
            })

            return response

        return render(request, 'django_we/errmsg.html', {'title': 'Error', 'errmsg': 'DJANGO_WE_BASE_COOKIE_FUNC Should Exists'})

    if hasattr(settings, 'DJANGO_WE_BASE_FUNC') and hasattr(settings.DJANGO_WE_BASE_FUNC, '__call__'):
        query_params = settings.DJANGO_WE_BASE_FUNC(code, final_state, access_info) or {}

    return redirect(furl(final_state).remove(access_info.keys()).add(access_info).remove(query_params.keys()).add(query_params).url)


@transaction.atomic
def userinfo_redirect(request):
    code = request.GET.get('code', '')
    state = request.GET.get('state', '')

    final_state = unquote_state(request, state)

    CFG = final_cfg(request, final_state)

    access_info = get_access_info(CFG['appID'], CFG['appsecret'], code)
    if 'errcode' in access_info:
        return redirect(get_oauth_redirect_url(final_oauth_uri(request, final_state), 'snsapi_userinfo', final_state))

    userinfo = get_userinfo(access_info.get('access_token', ''), access_info.get('openid', ''))
    if 'openid' not in userinfo:
        return redirect(get_oauth_redirect_url(final_oauth_uri(request, final_state), 'snsapi_userinfo', final_state))

    query_params = {}

    # Toset Cookie When ``DJANGO_WE_USERINFO_REDIRECT_SET_COOKIE = True``
    if hasattr(settings, 'DJANGO_WE_USERINFO_REDIRECT_SET_COOKIE') and getattr(settings, 'DJANGO_WE_USERINFO_REDIRECT_SET_COOKIE'):
        if hasattr(settings, 'DJANGO_WE_USERINFO_COOKIE_FUNC') and hasattr(settings.DJANGO_WE_USERINFO_COOKIE_FUNC, '__call__'):
            query_params, cookie_key, cookie_value = settings.DJANGO_WE_USERINFO_COOKIE_FUNC(code, final_state, access_info, userinfo) or {}

            response = redirect(furl(final_state).remove(userinfo.keys()).add(userinfo).remove(query_params.keys()).add(query_params).url)

            if hasattr(settings, 'DJANGO_WE_USERINFO_SET_COOKIE_FUNC') and hasattr(settings.DJANGO_WE_USERINFO_SET_COOKIE_FUNC, '__call__'):
                return settings.DJANGO_WE_USERINFO_SET_COOKIE_FUNC(code, final_state, access_info, userinfo, query_params, cookie_key, cookie_value)

            max_age = hasattr(settings, 'DJANGO_WE_COOKIE_MAX_AGE') and getattr(settings, 'DJANGO_WE_COOKIE_MAX_AGE') or 30 * 24 * 60 * 60  # 30d
            cookie_salt = hasattr(settings, 'DJANGO_WE_COOKIE_SALT') and getattr(settings, 'DJANGO_WE_COOKIE_SALT') or 'djwe'  # Salt for ``set_signed_cookie``

            response.set_signed_cookie(cookie_key, cookie_value, salt=cookie_salt, **{
                'max_age': max_age,
                'httponly': True,
            })

            return response

        return render(request, 'django_we/errmsg.html', {'title': 'Error', 'errmsg': 'DJANGO_WE_USERINFO_COOKIE_FUNC Should Exists'})

    if hasattr(settings, 'DJANGO_WE_USERINFO_FUNC') and hasattr(settings.DJANGO_WE_USERINFO_FUNC, '__call__'):
        query_params = settings.DJANGO_WE_USERINFO_FUNC(code, final_state, access_info, userinfo) or {}

    return redirect(furl(final_state).remove(userinfo.keys()).add(userinfo).remove(query_params.keys()).add(query_params).url)


def direct_base_redirect(request):
    code = request.GET.get('code', '')
    state = request.GET.get('state', '')

    final_state = unquote_state(request, state)

    CFG = final_cfg(request, final_state)

    access_info = get_access_info(CFG['appID'], CFG['appsecret'], code)
    if 'errcode' in access_info:
        return redirect(get_oauth_redirect_url(final_oauth_uri(request, final_state), 'snsapi_base', final_state, direct_redirect=True))

    return redirect(furl(final_state).remove(access_info.keys()).add(access_info).url)


def direct_userinfo_redirect(request):
    code = request.GET.get('code', '')
    state = request.GET.get('state', '')

    final_state = unquote_state(request, state)

    CFG = final_cfg(request, final_state)

    access_info = get_access_info(CFG['appID'], CFG['appsecret'], code)
    if 'errcode' in access_info:
        return redirect(get_oauth_redirect_url(final_oauth_uri(request, final_state), 'snsapi_userinfo', final_state, direct_redirect=True))

    userinfo = get_userinfo(access_info.get('access_token', ''), access_info.get('openid', ''))
    if 'openid' not in userinfo:
        return redirect(get_oauth_redirect_url(final_oauth_uri(request, final_state), 'snsapi_userinfo', final_state, direct_redirect=True))

    return redirect(furl(final_state).remove(userinfo.keys()).add(userinfo).url)


def we_share(request):
    redirect_url = ''

    if hasattr(settings, 'WECHAT_OAUTH2_REDIRECT_URL'):
        redirect_url = settings.WECHAT_OAUTH2_REDIRECT_URL

    if hasattr(settings, 'DJANGO_WE_SHARE_FUNC') and hasattr(settings.DJANGO_WE_SHARE_FUNC, '__call__'):
        redirect_url = settings.DJANGO_WE_SHARE_FUNC(request) or redirect_url

    if not redirect_url:
        return render(request, 'django_we/errmsg.html', {'title': 'Error', 'errmsg': 'Redirect URL Should Exists'})

    return redirect(redirect_url)


@auto_response
def we_jsapi_signature_api(request, state=None):
    if hasattr(settings, 'DJANGO_WE_ENABLED_OR_NOT') and not getattr(settings, 'DJANGO_WE_ENABLED_OR_NOT'):
        return {}
    CFG = final_cfg(request, state=state or 'jsapi_signature')
    return jsapi_signature_params(
        CFG['appID'],
        CFG['appsecret'],
        request.GET.get('url', '') or request.POST.get('url', ''),
        storage=redis_storage(request),
        token_fetched_func=token_fetched_func,
        ticket_fetched_func=ticket_fetched_func
    )


@auto_response
def we_access_token(request, state=None):
    fetch_type = request.GET.get('fetch_type', '')

    if fetch_type == 'refresh':
        fetch_func = refresh_access_token
    else:
        fetch_func = access_token

    CFG = final_cfg(request, state=state or 'access_token')

    return {
        'appid': CFG['appID'],
        'access_token': fetch_func(
            CFG['appID'],
            CFG['appsecret'],
            storage=redis_storage(request),
            token_fetched_func=token_fetched_func
        ),
    }


@transaction.atomic
@logit(body=True, res=True)
def we_callback(request):
    signature = request.GET.get('signature', '')
    timestamp = request.GET.get('timestamp', '')
    nonce = request.GET.get('nonce', '')
    echostr = request.GET.get('echostr', '')
    encrypt_type = request.GET.get('encrypt_type', '')
    msg_signature = request.GET.get('msg_signature', '')

    CFG = final_cfg(request, state='callback')

    # 校验签名
    if not check_callback_signature(CFG['token'], signature, timestamp, nonce):
        return HttpResponse()

    if request.method == 'GET':
        return HttpResponse(echostr)

    xml = request.body

    resp_xml = ''
    if hasattr(settings, 'DJANGO_WE_MESSAGE_CALLBACK_FUNC') and hasattr(settings.DJANGO_WE_MESSAGE_CALLBACK_FUNC, '__call__'):
        decrypted = msg.decrypt(CFG['appID'], token=CFG['token'], encodingaeskey=CFG['encodingaeskey'], post_data=xml, encrypt=None, msg_signature=msg_signature, timestamp=timestamp, nonce=nonce, xmltodict=True)
        resp_xml = settings.DJANGO_WE_MESSAGE_CALLBACK_FUNC(request, xml_to_dict(xml), decrypted or {}) or ''

    if resp_xml:
        resp_xml = msg.encrypt(CFG['appID'], token=CFG['token'], encodingaeskey=CFG['encodingaeskey'], resp_xml=resp_xml, nonce=nonce, timestamp=None, random_str=None)

    return HttpResponse(resp_xml or 'success')


@transaction.atomic
@logit(body=True, res=True)
def we_component_auth(request):
    signature = request.GET.get('signature', '')
    timestamp = request.GET.get('timestamp', '')
    nonce = request.GET.get('nonce', '')
    encrypt_type = request.GET.get('encrypt_type', '')
    msg_signature = request.GET.get('msg_signature', '')

    CFG = final_cfg(request, state='component_auth')

    # 校验签名
    if not check_callback_signature(CFG['token'], signature, timestamp, nonce):
        return HttpResponse()

    xml = request.body

    # 消息解密
    decrypted = msg.decrypt(CFG['appID'], token=CFG['token'], encodingaeskey=CFG['encodingaeskey'], post_data=xml, encrypt=None, msg_signature=msg_signature, timestamp=timestamp, nonce=nonce, xmltodict=True)

    # 获取 InfoType
    InfoType = decrypted.get('InfoType', '')  # unauthorized是取消授权，updateauthorized是更新授权，authorized是授权成功通知，component_verify_ticket

    # 当 InfoType 为 component_verify_ticket 时，进行保存 component_verify_ticket 的操作
    if InfoType == 'component_verify_ticket':
        # Set Component Verify Ticket into Redis
        set_component_verify_ticket(
            appid=CFG['appID'],
            secret=CFG['appsecret'],
            token=CFG['token'],
            encodingaeskey=CFG['encodingaeskey'],
            post_data=xml,
            encrypt=None,
            msg_signature=msg_signature,
            timestamp=timestamp,
            nonce=nonce,
            storage=redis_storage(request),
        )

        # Set Component Verify Ticket into MySQL
        component_verify_ticket_push_func(CFG['appID'], CFG['appsecret'], decrypted)

    resp_xml = ''
    if hasattr(settings, 'DJANGO_WE_COMPONENT_AUTH_FUNC') and hasattr(settings.DJANGO_WE_COMPONENT_AUTH_FUNC, '__call__'):
        resp_xml = settings.DJANGO_WE_COMPONENT_AUTH_FUNC(request, xml_to_dict(xml), decrypted or {}) or ''

    if resp_xml:
        resp_xml = msg.encrypt(CFG['appID'], token=CFG['token'], encodingaeskey=CFG['encodingaeskey'], resp_xml=resp_xml, nonce=nonce, timestamp=None, random_str=None)

    return HttpResponse(resp_xml or 'success')


@transaction.atomic
@logit(body=True, res=True)
def we_component_callback(request, appid=None):
    signature = request.GET.get('signature', '')
    timestamp = request.GET.get('timestamp', '')
    nonce = request.GET.get('nonce', '')
    encrypt_type = request.GET.get('encrypt_type', '')
    msg_signature = request.GET.get('msg_signature', '')

    CFG = final_cfg(request, state='component_callback')

    # 校验签名
    if not check_callback_signature(CFG['token'], signature, timestamp, nonce):
        return HttpResponse()

    xml = request.body

    resp_xml = ''
    if hasattr(settings, 'DJANGO_WE_COMPONENT_CALLBACK_FUNC') and hasattr(settings.DJANGO_WE_COMPONENT_CALLBACK_FUNC, '__call__'):
        decrypted = msg.decrypt(CFG['appID'], token=CFG['token'], encodingaeskey=CFG['encodingaeskey'], post_data=xml, encrypt=None, msg_signature=msg_signature, timestamp=timestamp, nonce=nonce, xmltodict=True)
        resp_xml = settings.DJANGO_WE_COMPONENT_CALLBACK_FUNC(request, appid, xml_to_dict(xml), decrypted or {}) or ''

    if resp_xml:
        resp_xml = msg.encrypt(CFG['appID'], token=CFG['token'], encodingaeskey=CFG['encodingaeskey'], resp_xml=resp_xml, nonce=nonce, timestamp=None, random_str=None)

    return HttpResponse(resp_xml or 'success')


@logit(body=True, res=True)
def we_component_preauth_callback(request):
    auth_code = request.GET.get('auth_code', '')

    CFG = final_cfg(request, state='component_preauth_callback')

    authorizer_access_token, authorizer_appid = initial_authorizer_access_token(
        component_appid=CFG['appID'],
        component_secret=CFG['appsecret'],
        auth_code=auth_code,
        storage=redis_storage(request),
        token_fetched_func=component_token_fetched_func,
        auth_token_fetched_func=component_auth_token_fetched_func,
        with_authorizer_appid=True
    )

    if hasattr(settings, 'DJANGO_WE_COMPONENT_CALLBACK_TEMPLATE'):
        redirect_tpl = settings.DJANGO_WE_COMPONENT_CALLBACK_TEMPLATE
        if redirect_tpl:
            return render(request, redirect_tpl, {'authorizer_appid': authorizer_appid})

    return render(request, 'django_we/errmsg.html', {'title': 'Success', 'errmsg': 'Component Auth Success'})


@auto_response
def we_qrcode_url(request, state=None):
    authorizer_appid = request.GET.get('authorizer_appid', '')
    action_name = request.GET.get('action_name', 'QR_SCENE')
    scene_id = int(request.GET.get('scene_id', 0))
    scene_str = request.GET.get('scene_str', '')
    expire_seconds = int(request.GET.get('expire_seconds', 2592000))

    CFG = final_cfg(request, state=state or 'qrcode_url')

    if state == 'component':
        token = authorizer_access_token(
            component_appid=CFG['appID'],
            component_secret=CFG['appsecret'],
            authorizer_appid=authorizer_appid,
            storage=redis_storage(request),
            token_fetched_func=component_token_fetched_func,
            auth_token_fetched_func=component_auth_token_fetched_func
        )
    else:
        token = access_token(
            CFG['appID'],
            CFG['appsecret'],
            storage=redis_storage(request),
            token_fetched_func=token_fetched_func
        )

    return {
        'qrinfo': qrcode_create(
            action_name=action_name,
            scene_id=scene_id,
            scene_str=scene_str,
            expire_seconds=expire_seconds,
            appid=CFG['appID'],
            secret=CFG['appsecret'],
            token=token,
            storage=redis_storage(request)
        ),
    }



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-05-02 07:48
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ComponentAuthTokenRefreshLogInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.BooleanField(db_index=True, default=True, help_text='Status', verbose_name='status')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='Create Time', verbose_name='created_at')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='Update Time', verbose_name='updated_at')),
                ('component_appid', models.CharField(blank=True, db_index=True, help_text='Component APPID', max_length=32, null=True, verbose_name='component_appid')),
                ('component_secret', models.CharField(blank=True, help_text='Component Secret', max_length=32, null=True, verbose_name='component_secret')),
                ('authorizer_appid', models.CharField(blank=True, db_index=True, help_text='Authorizer APPID', max_length=32, null=True, verbose_name='authorizer_appid')),
                ('component_authorizer_access_info', jsonfield.fields.JSONField(blank=True, help_text='Component Authorizer Access Info', null=True, verbose_name='component_authorizer_access_info')),
            ],
            options={
                'verbose_name': 'componentauthtokenrefreshloginfo',
                'verbose_name_plural': 'componentauthtokenrefreshloginfo',
            },
        ),
        migrations.CreateModel(
            name='ComponentTokenRefreshLogInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.BooleanField(db_index=True, default=True, help_text='Status', verbose_name='status')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='Create Time', verbose_name='created_at')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='Update Time', verbose_name='updated_at')),
                ('component_appid', models.CharField(blank=True, db_index=True, help_text='Component APPID', max_length=32, null=True, verbose_name='component_appid')),
                ('component_secret', models.CharField(blank=True, help_text='Component Secret', max_length=32, null=True, verbose_name='component_secret')),
                ('component_access_info', jsonfield.fields.JSONField(blank=True, help_text='Component Access Info', null=True, verbose_name='component_access_info')),
            ],
            options={
                'verbose_name': 'componenttokenrefreshloginfo',
                'verbose_name_plural': 'componenttokenrefreshloginfo',
            },
        ),
        migrations.CreateModel(
            name='TicketRefreshLogInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.BooleanField(db_index=True, default=True, help_text='Status', verbose_name='status')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='Create Time', verbose_name='created_at')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='Update Time', verbose_name='updated_at')),
                ('appid', models.CharField(blank=True, db_index=True, help_text='APPID', max_length=32, null=True, verbose_name='appid')),
                ('secret', models.CharField(blank=True, help_text='Secret', max_length=32, null=True, verbose_name='secret')),
                ('ticket_type', models.CharField(blank=True, help_text='Ticket Type', max_length=8, null=True, verbose_name='ticket_type')),
                ('ticket_info', jsonfield.fields.JSONField(blank=True, help_text='Ticket Info', null=True, verbose_name='ticket_info')),
            ],
            options={
                'verbose_name': 'ticketrefreshloginfo',
                'verbose_name_plural': 'ticketrefreshloginfo',
            },
        ),
        migrations.CreateModel(
            name='TokenRefreshLogInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.BooleanField(db_index=True, default=True, help_text='Status', verbose_name='status')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='Create Time', verbose_name='created_at')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='Update Time', verbose_name='updated_at')),
                ('appid', models.CharField(blank=True, db_index=True, help_text='APPID', max_length=32, null=True, verbose_name='appid')),
                ('secret', models.CharField(blank=True, help_text='Secret', max_length=32, null=True, verbose_name='secret')),
                ('access_info', jsonfield.fields.JSONField(blank=True, help_text='Access Info', null=True, verbose_name='access_info')),
            ],
            options={
                'verbose_name': 'tokenrefreshloginfo',
                'verbose_name_plural': 'tokenrefreshloginfo',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from setuptools import setup


version = '1.5.5'


setup(
    name='django-we',
    version=version,
    keywords='django-we',
    description='Django WeChat OAuth2/Share/Token API',
    long_description=open('README.rst').read(),

    url='https://github.com/django-xxx/django-we',

    author='Hackathon',
    author_email='kimi.huang@brightcells.com',

    packages=['django_we'],
    py_modules=[],
    install_requires=['django-admin>=1.2.4', 'django-detect', 'django-json-response', 'django-logit', 'django-models-ext', 'furl', 'jsonfield', 'pywe-component-authorizer-token>=1.1.1', 'pywe-component-ticket', 'pywe-decrypt>=1.1.3', 'pywe-jssdk>=1.1.0', 'pywe-oauth>=1.0.5', 'pywe-qrcode', 'pywe-sign>=1.0.7', 'pywe-storage', 'pywe-token>=1.2.1', 'pywe-xml'],
    include_package_data=True,

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Office/Business :: Financial :: Spreadsheet',
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.22 on 2019-08-20 04:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_we', '0003_subscribeuserinfo'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscribeuserinfo',
            name='extraid',
            field=models.CharField(blank=True, db_index=True, help_text='ExtraID', max_length=32, null=True, verbose_name='extraid'),
        ),
        migrations.AlterUniqueTogether(
            name='subscribeuserinfo',
            unique_together=set([('extraid', 'unionid', 'openid')]),
        ),
    ]
